@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 # -----------------------------------------------------------------------
-# Copyright (c) 2009  Jendrik Seipp
+# Copyright (c) 2009, 2014  Jendrik Seipp
 #
 # RedNotebook is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -25,15 +25,9 @@ from rednotebook.util import filesystem
 
 
 def delete_comment(line):
-    '''
-    delete comment, do not alter the line,
-    if no comment sign is found
-    '''
-    comment_pos = line.find('#')
-    if comment_pos >= 0:
-        return line[:comment_pos]
-    else:
-        return line
+    if line.startswith('#'):
+        return u''
+    return line
 
 
 
@@ -41,7 +35,7 @@ class Config(dict):
     def __init__(self, config_file):
         dict.__init__(self)
 
-        self.file = config_file
+        self.filename = config_file
 
         self.obsolete_keys = [
             u'useGTKMozembed', u'useWebkit', u'LD_LIBRARY_PATH',
@@ -51,9 +45,8 @@ class Config(dict):
         # Allow changing the value of portable only in default.cfg
         self.suppressed_keys = ['portable', 'user_dir']
 
-        self.update(self._read_file(self.file))
-
-        self.set_default_values()
+        self.update(self._read_file(self.filename))
+        self.save_state()
 
 
     def save_state(self):
@@ -61,71 +54,43 @@ class Config(dict):
         self.old_config = self.copy()
 
 
-    def set_default_values(self):
-        '''
-        Sets some default values that are not automatically set so that
-        they appear in the config file
-        '''
-        #self.read('export_date_format', '%A, %x')
+    def _read_file(self, filename):
+        content = filesystem.read_file(filename)
 
-
-    def _read_file(self, file):
-
-        key_value_pairs = []
-
-        content = filesystem.read_file(file)
-        if not content:
-            return {}
-
-        lines = content.split('\n')
-
-        # delete comments
-        key_value_pairs = map(lambda line: delete_comment(line), lines)
-
-        #delete whitespace
-        key_value_pairs = map(unicode.strip, key_value_pairs)
-
-        #delete empty lines
-        key_value_pairs = filter(bool, key_value_pairs)
+        # Delete comments and whitespace.
+        lines = [delete_comment(line.strip()) for line in content.splitlines()]
 
         dictionary = {}
 
-        #read keys and values
-        for key_value_pair in key_value_pairs:
-            if '=' in key_value_pair:
-                try:
-                    # Delete whitespace around =
-                    pair = key_value_pair.partition('=')[::2]
-                    key, value = map(unicode.strip, pair)
+        for line in lines:
+            if '=' not in line:
+                continue
+            pair = line.partition('=')[::2]
+            key, value = [s.strip() for s in pair]
+            # Skip obsolete keys to prevent rewriting them to disk.
+            if key in self.obsolete_keys:
+                continue
 
-                    # Do not add obsolete keys -> they will not be rewritten
-                    # to disk
-                    if key in self.obsolete_keys:
-                        continue
+            try:
+                value = int(value)
+            except ValueError:
+                pass
 
-                    try:
-                        #Save value as int if possible
-                        dictionary[key] = int(value)
-                    except ValueError:
-                        dictionary[key] = value
+            dictionary[key] = value
 
-                except Exception:
-                    msg = 'The line "%s" in the config file contains errors'
-                    logging.error(msg % key_value_pair)
         return dictionary
 
 
     def read(self, key, default):
         if key in self:
-            return self.get(key)
-        else:
-            self[key] = default
-            return default
+            return self[key]
+        self[key] = default
+        return default
 
 
     def read_list(self, key, default):
         '''
-        Reads the string corresponding to key and converts it to a list
+        Read the string corresponding to key and convert it to a list.
 
         alpha,beta gamma;delta -> ['alpha', 'beta', 'gamma', 'delta']
 
@@ -133,23 +98,13 @@ class Config(dict):
         '''
         string = self.read(key, default)
         string = unicode(string)
-        if not string:
-            return []
 
-        # Try to convert the string to a list
         separators = [',', ';']
         for separator in separators:
             string = string.replace(separator, ' ')
 
-        list = string.split()
-
-        # Remove whitespace
-        list = map(unicode.strip, list)
-
-        # Remove empty items
-        list = filter(bool, list)
-
-        return list
+        strings = [s.strip() for s in string.split()]
+        return [s for s in strings if s]
 
 
     def write_list(self, key, list):
@@ -164,18 +119,17 @@ class Config(dict):
         if not self.changed():
             return
 
-        content = ''
+        lines = []
         for key, value in sorted(self.iteritems()):
             if key not in self.suppressed_keys:
-                content += ('%s=%s\n' % (key, value))
+                lines.append('%s=%s' % (key, value))
 
         try:
-            filesystem.make_directory(os.path.dirname(self.file))
-            filesystem.write_file(self.file, content)
+            filesystem.make_directory(os.path.dirname(self.filename))
+            filesystem.write_file(self.filename, '\n'.join(lines))
         except IOError:
             logging.error('Configuration could not be saved. Please check '
                           'your permissions')
-            return
-
-        logging.info('Configuration has been saved to %s' % self.file)
-        self.save_state()
+        else:
+            logging.info('Configuration has been saved to %s' % self.filename)
+            self.save_state()
