@@ -17,16 +17,15 @@
 # 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 # -----------------------------------------------------------------------
 
-import sys
-import os
 import datetime
 import logging
+import os
+import sys
 
-import gtk
 import gobject
+import gtk
 import pango
 
-
 from rednotebook.gui.menu import MainMenuBar
 from rednotebook.gui.options import OptionsManager
 from rednotebook.gui import customwidgets
@@ -154,8 +153,7 @@ class MainWindow(object):
 
         self.load_values_from_config()
 
-        if not self.journal.start_minimized:
-            self.main_frame.show()
+        self.main_frame.show()
 
         self.options_manager = OptionsManager(self)
         self.export_assistant = ExportAssistant(self.journal)
@@ -224,6 +222,8 @@ class MainWindow(object):
         #   self.day_text_field.day_text_view.add_accelerator(signal, self.accel_group,
         #                   ord(key), gtk.gdk.CONTROL_MASK, gtk.ACCEL_VISIBLE)
 
+        self.main_frame.connect('key-press-event', self._on_key_press_event)
+
         shortcuts = [
             (self.back_one_day_button, 'clicked', '<Ctrl>Page_Up'),
             (self.today_button, 'clicked', '<Alt>Home'),
@@ -232,12 +232,16 @@ class MainWindow(object):
             #(self.builder.get_object('redo_menuitem'), 'activate', '<Ctrl>y'),
             #(self.builder.get_object('options_menuitem'), 'activate', '<Ctrl><Alt>p'),
         ]
-
         for button, signal, shortcut in shortcuts:
             (keyval, mod) = gtk.accelerator_parse(shortcut)
             button.add_accelerator(signal, self.accel_group,
                                    keyval, mod, gtk.ACCEL_VISIBLE)
 
+    def _on_key_press_event(self, widget, event):
+        # Exit fullscreen mode with ESC.
+        if event.keyval == gtk.keysyms.Escape and self.is_fullscreen:
+            self.toggle_fullscreen()
+
 
     # TRAY-ICON / CLOSE --------------------------------------------------------
 
@@ -356,7 +360,6 @@ class MainWindow(object):
         size_group.add_widget(edit_button)
         size_group.add_widget(preview_button)
 
-        # Do not forget to update the text in editor and preview respectively
         if preview:
             # Enter preview mode
             edit_scroll.hide()
@@ -373,7 +376,6 @@ class MainWindow(object):
             edit_button.hide()
 
         self.template_manager.set_template_menu_sensitive(not preview)
-        # Disable insert shortcuts in preview mode.
         self.insert_actiongroup.set_sensitive(not preview)
         self.format_actiongroup.set_sensitive(not preview)
         self.insert_button.set_sensitive(not preview)
@@ -444,11 +446,9 @@ class MainWindow(object):
     def toggle_fullscreen(self):
         if self.is_fullscreen:
             self.main_frame.unfullscreen()
-            self.menubar.show()
             self.is_fullscreen = False
         else:
             self.main_frame.fullscreen()
-            self.menubar.hide()
             self.is_fullscreen = True
 
     def on_back_one_day_button_clicked(self, widget):
@@ -871,9 +871,6 @@ class Calendar(object):
             self.calendar.unmark_day(day_number)
 
     def set_month(self, month):
-        #month_days = dates.get_number_of_days(month.year_number, month.month_number)
-        #for day_number in range(1, month_days + 1):
-        #   self.set_day_edited(day_number, False)
         self.calendar.clear_marks()
         for day_number, day in month.days.items():
             self.set_day_edited(day_number, not day.empty)
