@@ -265,7 +265,7 @@ class OptionsManager(object):
         # If this option is activated on a system without a systray, the
         # application keeps on running in the background after it has been
         # closed. The option can still be activated in the configuration file.
-        if filesystem.HAS_TRAY:
+        if filesystem.has_system_tray():
             self.options.append(TickOption(_('Close to system tray'), 'closeToTray',
                 tooltip=_('Closing the window will send RedNotebook to the tray')))
 
@@ -294,7 +294,7 @@ class OptionsManager(object):
                        tooltip=_('Comma-separated font names')),
             DateFormatOption(_('Date/Time format'), 'dateTimeString'),
             TextOption(_('Exclude from cloud'), 'cloudIgnoreList',
-                       tooltip=_('Do not show these comma separated words in the word cloud')),
+                       tooltip=_('Do not show these comma separated words and #tags in the clouds')),
             TextOption(_('Include small words in cloud'), 'cloudIncludeList',
                        tooltip=_('Allow these words with 4 letters or less')),
         ])
