@@ -23,31 +23,25 @@ PyGTKCodeBuffer by Hannes Matuschek (http://code.google.com/p/pygtkcodebuffer/).
 """
 
 import gtk
-import sys
-import os.path
 import pango
+
+import collections
 import logging
+import os
 import re
+import sys
 
 if __name__ == '__main__':
     sys.path.insert(0, os.path.abspath("./../../"))
     logging.getLogger('').setLevel(logging.DEBUG)
 
+from rednotebook.data import HASHTAG
 from rednotebook.external import txt2tags
 from rednotebook.gui.browser import HtmlView
 from rednotebook.util import markup
 
 
-class Tag(object):
-    def __init__(self, start, end, tagname, rule):
-        self.start = start
-        self.end = end
-        self.name = tagname
-        self.rule = rule
-
-    def list(self):
-        return [self.start, self.end, self.name]
-
+Tag = collections.namedtuple('Tag', ['start', 'end', 'name', 'rule'])
 
 class TagGroup(list):
     @property
@@ -58,58 +52,42 @@ class TagGroup(list):
     def max_end(self):
         return max([tag.end for tag in self], key=lambda i: i.get_offset()).copy()
 
-    def sort(self):
-        key = lambda tag: (tag.start.get_offset(), -tag.end.get_offset(), tag.name)
-        list.sort(self, key=key)
-
     @property
     def rule(self):
         if len(self) == 0:
             return 'NO ITEM'
         return self[0].rule
 
-    def list(self):
-        return map(lambda g: g.list(), self)
-
-    #def __cmp__(self, other):
-    #    return cmp((self.min_start, other.min_start))
-
 
 class Pattern(object):
     """
     A pattern object allows a regex-pattern to have
     subgroups with different formatting
     """
-    def __init__(self, pattern, group_tag_pairs, regex=None, flags="",
+    def __init__(self, pattern_or_regex, group_tag_pairs, flags="",
                  name='unnamed'):
         self.name = name
-
-        # assemble re-flag
-        flags += "ML"
-        flag = 0
-
-        for char in flags:
-            if char == 'M': flag |= re.M
-            if char == 'L': flag |= re.L
-            if char == 'S': flag |= re.S
-            if char == 'I': flag |= re.I
-            if char == 'U': flag |= re.U
-            if char == 'X': flag |= re.X
-
-        if regex:
-            self._regexp = regex
-        else:
-            # compile re
-            try:
-                self._regexp = re.compile(pattern, flag)
-            except re.error, e:
-                raise Exception("Invalid regexp \"%s\": %s" % (pattern, e))
-
         self.group_tag_pairs = group_tag_pairs
 
+        if isinstance(pattern_or_regex, basestring):
+            # assemble re-flag
+            # TODO: Is this needed?
+            flags += "ML"
+            flag = 0
+
+            for char in flags:
+                assert char in 'MLSIUX'
+                flag |= getattr(re, char)
+
+            self._regexp = re.compile(pattern_or_regex, flag)
+        else:
+            assert not flags, flags
+            self._regexp = pattern_or_regex
+
     def __call__(self, txt, start, end):
         m = self._regexp.search(txt)
-        if not m: return None
+        if not m:
+            return None
 
         tags = TagGroup()
 
@@ -118,8 +96,10 @@ class Pattern(object):
             if not group_matched:
                 continue
             mstart, mend = m.start(group), m.end(group)
-            s = start.copy(); s.forward_chars(mstart)
-            e = start.copy(); e.forward_chars(mend)
+            s = start.copy()
+            s.forward_chars(mstart)
+            e = start.copy()
+            e.forward_chars(mend)
             tag = Tag(s, e, tag_name, self.name)
             tags.append(tag)
 
@@ -150,28 +130,21 @@ class MarkupDefinition(object):
                 tags = rule(subtext, tags.max_end, end)
 
         tag_groups.sort(key=lambda g: (g.min_start.get_offset(), -g.max_end.get_offset()))
-
         return tag_groups
 
 
 class MarkupBuffer(gtk.TextBuffer):
+    OVERLAPS = ['bold', 'italic', 'underline', 'strikethrough',
+                'highlight', 'list', 'numlist']
+
     def __init__(self, table=None, lang=None, styles={}):
         gtk.TextBuffer.__init__(self, table)
-
-        # update styles with user-defined
+        self._lang_def = lang
         self.styles = styles
 
         # create tags
         for name, props in self.styles.items():
-            style = {}
-            style.update(props)
-            self.create_tag(name, **style)
-
-        # store lang-definition
-        self._lang_def = lang
-
-        self.overlaps = ['bold', 'italic', 'underline', 'strikethrough',
-                         'highlight', 'list', 'numlist']
+            self.create_tag(name, **props)
 
         self.connect_after("insert-text", self._on_insert_text)
         self.connect_after("delete-range", self._on_delete_range)
@@ -185,49 +158,37 @@ class MarkupBuffer(gtk.TextBuffer):
 
     def get_slice(self, start, end):
         """
-        We have to search for the regexes in utf-8 text
+        We have to search for the regexes in utf-8 text.
         """
-        slice_text = gtk.TextBuffer.get_slice(self, start, end)
-        slice_text = slice_text.decode('utf-8')
-        return slice_text
+        return gtk.TextBuffer.get_slice(self, start, end).decode('utf-8')
 
     def _on_insert_text(self, buf, it, text, length):
         end = it.copy()
         start = it.copy()
         start.backward_chars(length)
-
         self.update_syntax(start, end)
 
     def _on_delete_range(self, buf, start, end):
-        start = start.copy()
-
-        self.update_syntax(start, start)
+        # Copy the iters here to keep them valid for spellchecking's replacements.
+        self.update_syntax(start.copy(), start.copy())
 
     def remove_all_syntax_tags(self, start, end):
         """
-        Do not remove the gtkspell highlighting
+        Do not remove the gtkspell highlighting.
         """
         for style in self.styles:
             self.remove_tag_by_name(style, start, end)
 
     def apply_tags(self, tags):
-        for mstart, mend, tagname in tags.list():
+        for mstart, mend, tagname, rule in tags:
             # apply tag
             self.apply_tag_by_name(tagname, mstart, mend)
 
     def update_syntax(self, start, end):
-        """Use two categories of rules: one-line and multiline
-
-        Before running multiline rules: Check if e.g. - is present in changed
-        string.
-        """
         # Just update from the start of the first edited line
         # to the end of the last edited line, because we can
-        # guarantee that there's no multiline rule
-        start_line_number = start.get_line()
-        start_line_iter = self.get_iter_at_line(start_line_number)
-        start = start_line_iter
-
+        # guarantee that there's no multiline rule.
+        start = self.get_iter_at_line(start.get_line())
         end.forward_to_line_end()
 
         # remove all tags from start to end
@@ -236,7 +197,6 @@ class MarkupBuffer(gtk.TextBuffer):
         tag_groups = self._lang_def(self, start, end)
 
         min_start = start.copy()
-
         for tags in tag_groups:
             if tags.rule == 'highlight':
                 self.apply_tags(tags)
@@ -245,49 +205,46 @@ class MarkupBuffer(gtk.TextBuffer):
                 # min_start is left or equal to tags.min_start
                 self.apply_tags(tags)
 
-                if tags.rule in self.overlaps:
+                if tags.rule in self.OVERLAPS:
                     min_start = tags.min_start
                 else:
                     min_start = tags.max_end
 
 
-# additional style definitions:
-# the update_syntax() method of CodeBuffer allows you to define new and modify
-# already defined styles. Think of it like CSS.
-styles = {  'bold':             {'weight': pango.WEIGHT_BOLD},
-            'italic':           {   # Just to be sure we leave this in
-                                    'style': pango.STYLE_ITALIC,
-                                    # The font:Italic is actually needed
-                                    #'font': 'Italic',
-                                    },
-            'underline':        {'underline': pango.UNDERLINE_SINGLE},
-            'strikethrough':    {'strikethrough': True},
-            'gray':             {'foreground': 'gray'},
-            'red':              {'foreground': 'red'},
-            'green':            {'foreground': 'darkgreen'},
-            'raw':              {'font': 'Oblique'},
-            'verbatim':         {'font': 'monospace'},
-            'tagged':           {},
-            'link':             {'foreground': 'blue',
-                                'underline': pango.UNDERLINE_SINGLE,},
-            'highlight':        {'background': 'yellow'},
-            'quote':            {'background': 'gray'},
-            'tablehead':        {'background': markup.TABLE_HEAD_BG},
-            'tablerow':         {'background': '#eee'}
-            }
+styles = {
+    'bold':             {'weight': pango.WEIGHT_BOLD},
+    'italic':           {'style': pango.STYLE_ITALIC},
+    'underline':        {'underline': pango.UNDERLINE_SINGLE},
+    'strikethrough':    {'strikethrough': True},
+    'gray':             {'foreground': 'gray'},
+    'red':              {'foreground': 'red'},
+    'green':            {'foreground': 'darkgreen'},
+    'raw':              {'font': 'Oblique'},
+    'verbatim':         {'font': 'monospace'},
+    'tagged':           {},
+    'link':             {'foreground': 'blue',
+                         'underline': pango.UNDERLINE_SINGLE},
+    'highlight':        {'background': 'yellow'},
+    'quote':            {'background': 'gray'},
+    'tablehead':        {'background': markup.TABLE_HEAD_BG},
+    'tablerow':         {'background': '#eee'},
+    'formula':          {'style': pango.STYLE_ITALIC, 'family': 'serif'}
+}
+
 def add_header_styles():
     sizes = [
-            pango.SCALE_XX_LARGE,
-            pango.SCALE_X_LARGE,
-            pango.SCALE_LARGE,
-            pango.SCALE_MEDIUM,
-            pango.SCALE_SMALL,
-            ]
+        pango.SCALE_XX_LARGE,
+        pango.SCALE_X_LARGE,
+        pango.SCALE_LARGE,
+        pango.SCALE_MEDIUM,
+        pango.SCALE_SMALL,
+    ]
     for level, size in enumerate(sizes):
         style = {'weight': pango.WEIGHT_ULTRABOLD,
                 'scale': size}
-        name = 'title%s' % (level+1)
+        name = 'title%s' % (level + 1)
         styles[name] = style
+
 add_header_styles()
 
 # Syntax definition
@@ -307,8 +264,8 @@ def get_pattern(char, style):
     return Pattern(regex, group_style_pairs, name=style)
 
 
-list    = Pattern(r"^ *(\-) [^ ].*$", [(1, 'red'), (1, 'bold')], name='list')
-numlist = Pattern(r"^ *(\+) [^ ].*$", [(1, 'red'), (1, 'bold')], name='numlist')
+bullet_list = Pattern(r"^ *(\-) [^ ].*$", [(1, 'red'), (1, 'bold')], name='list')
+number_list = Pattern(r"^ *(\+) [^ ].*$", [(1, 'red'), (1, 'bold')], name='numlist')
 
 comment = Pattern(r'^(\%.*)$', [(1, 'gray')])
 
@@ -318,33 +275,27 @@ title_patterns = []
 title_style = [(1, 'gray'), (3, 'gray'), (4, 'gray')]
 titskel = r'^ *(%s)(%s)(\1)(\[[\w-]*\])?\s*$'
 for level in range(1, 6):
-    title_pattern    = titskel % ('[=]{%s}'%(level),'[^=]|[^=].*[^=]')
-    numtitle_pattern = titskel % ('[+]{%s}'%(level),'[^+]|[^+].*[^+]')
+    normal_title_pattern = titskel % ('[=]{%s}' % (level), '[^=]|[^=].*[^=]')
+    number_title_pattern = titskel % ('[+]{%s}' % (level), '[^+]|[^+].*[^+]')
     style_name = 'title%s' % level
-    title = Pattern(title_pattern, title_style + [(2, style_name)])
-    numtitle = Pattern(numtitle_pattern, title_style + [(2, style_name)])
-    title_patterns += [title, numtitle]
+    normal_title = Pattern(normal_title_pattern, title_style + [(2, style_name)])
+    number_title = Pattern(number_title_pattern, title_style + [(2, style_name)])
+    title_patterns += [normal_title, number_title]
 
 linebreak = Pattern(r'(%s)' % markup.REGEX_LINEBREAK, [(1, 'gray')])
 
 # pic [""/home/user/Desktop/RedNotebook pic"".png]
-# \w = [a-zA-Z0-9_]
-# Added ":-" for "file://5-5.jpg"
-# filename = One char or two chars with possibly whitespace in the middle
-#filename = r'\S[\w\s_,.+%$#@!?+~/-:-\(\)]*\S|\S'
-filename = r'\S.*?\S|\S'
-ext = r'(png|jpe?g|gif|eps|bmp)'
-pic = Pattern(r'(\["")(%s)("")(\.%s)(\?\d+)?(\])' % (filename, ext),
+pic = Pattern(markup.REGEX_PIC.pattern,
         [(1, 'gray'), (2, 'green'), (3, 'gray'), (4, 'green'), (5, 'gray'), (6, 'gray')], flags='I')
 
-# named link on hdd [my file.txt ""file:///home/user/my file.txt""]
+# named local link [my file.txt ""file:///home/user/my file.txt""]
 # named link in web [heise ""http://heise.de""]
-named_link = Pattern(r'(\[)(.*?)\s("")(\S.*?\S)(""\])',
-        [(1, 'gray'), (2, 'link'), (3, 'gray'), (4, 'gray'), (5, 'gray')], flags='LI')
+named_link = Pattern(markup.REGEX_NAMED_LINK,
+        [(1, 'gray'), (2, 'link'), (3, 'gray'), (4, 'gray'), (5, 'gray')])
 
 # link http://heise.de
 # Use txt2tags link guessing mechanism by setting regex explicitly
-link = Pattern('USEREGEX', [(0, 'link')], regex=bank['link'], name='link')
+link = Pattern(bank['link'], [(0, 'link')], name='link')
 
 # We do not support multiline regexes
 #blockverbatim = Pattern(r'^(```)\s*$\n(.*)$\n(```)\s*$', [(1, 'gray'), (2, 'verbatim'), (3, 'gray')])
@@ -354,27 +305,33 @@ quote = Pattern(r'^\t+(.*)$', [(1, 'quote')])
 table_head = Pattern(r'^ *(\|\| .*)', [(1, 'tablehead')])
 table_row = Pattern(r'^ *(\| .*)', [(1, 'tablerow')])
 
+formula = Pattern(r'(\\\(|\\\[|\$\$)(.+?)(\\\)|\\\]|\$\$)', [(1, 'gray'), (2, 'formula'), (3, 'gray')])
+
+hashtag = Pattern(HASHTAG, [(2, 'red'), (3, 'red')])
+
 
 patterns = [
-        get_pattern('\*', 'bold'),
-        get_pattern('_', 'underline'),
-        get_pattern('/', 'italic'),
-        get_pattern('-', 'strikethrough'),
-        list,
-        numlist,
-        comment,
-        line,
-        get_pattern('"', 'raw'),
-        get_pattern('`', 'verbatim'),
-        get_pattern("'", 'tagged'),
-        linebreak,
-        pic,
-        named_link,
-        link,
-        quote,
-        table_head,
-        table_row,
-        ] + title_patterns
+    get_pattern('\*', 'bold'),
+    get_pattern('_', 'underline'),
+    get_pattern('/', 'italic'),
+    get_pattern('-', 'strikethrough'),
+    bullet_list,
+    number_list,
+    comment,
+    line,
+    get_pattern('"', 'raw'),
+    get_pattern('`', 'verbatim'),
+    get_pattern("'", 'tagged'),
+    linebreak,
+    pic,
+    named_link,
+    link,
+    quote,
+    table_head,
+    table_row,
+    formula,
+    hashtag,
+] + title_patterns
 
 
 def get_highlight_buffer():
@@ -446,7 +403,7 @@ www.heise.de, andy@web.de
 
     def change_text(widget):
         html = markup.convert(widget.get_text(widget.get_start_iter(),
-                              widget.get_end_iter()), 'xhtml')
+                              widget.get_end_iter()), 'xhtml', '/tmp')
 
         html_editor.load_html(html)
         html_editor.highlight(search_text)
@@ -459,7 +416,7 @@ www.heise.de, andy@web.de
     win.add(vbox)
     scr.add(gtk.TextView(buff))
 
-    win.set_default_size(900,1000)
+    win.set_default_size(900, 1000)
     win.set_position(gtk.WIN_POS_CENTER)
     win.show_all()
     win.connect("destroy", lambda w: gtk.main_quit())
