@@ -29,18 +29,16 @@ try:
 except ImportError:
     from rednotebook.external import argparse
 
-from rednotebook.util import filesystem
 
-
-version = '1.8.1'
+version = '1.10.1'
 author = 'Jendrik Seipp'
-authorMail = 'jendrikseipp@web.de'
+author_mail = 'jendrikseipp@web.de'
 url = 'http://rednotebook.sourceforge.net'
 answers_url = 'https://answers.launchpad.net/rednotebook'
 translation_url = 'https://translations.launchpad.net/rednotebook/'
 bug_url = 'https://bugs.launchpad.net/rednotebook/+filebug'
 
-developers = ['Jendrik Seipp <jendrikseipp@web.de>',
+developers = ['%(author)s <%(author_mail)s>' % locals(),
               '',
               'Contributors:',
               'Alistair Marshall <thatscottishengineer@gmail.com>']
@@ -88,9 +86,6 @@ def get_commandline_parser():
         formatter_class=argparse.RawTextHelpFormatter)
     parser.add_argument('--version', action='version',
                         version='RedNotebook %s' % version)
-    if filesystem.HAS_TRAY:
-        parser.add_argument('-m', '--minimized', dest='minimized', action='store_true',
-                            help='start mimimized to system tray')
     parser.add_argument('--date', dest='start_date',
                         help='load specified date (format YYYY-MM-DD only)')
     parser.add_argument('journal', nargs='?', help=journal_path_help)
@@ -98,8 +93,6 @@ def get_commandline_parser():
 
 
 tags = _('Tags')
-work = _('Work')
-free_time = _('Free time')
 todo = _('Todo')
 done = _('Done')
 rtm = _('Remember the milk')
