@@ -178,14 +178,6 @@ class Journal:
 
         logging.info('Running in portable mode: %s' % self.dirs.portable)
 
-        # Allow starting minimized to tray
-        # When we start minimized we have to set the tray icon visible
-        self.start_minimized = filesystem.HAS_TRAY and args.minimized
-        if not filesystem.HAS_TRAY:
-            self.config['closeToTray'] = 0
-        elif self.start_minimized:
-            self.config['closeToTray'] = 1
-
         self.month = None
         self.date = None
         self.months = {}
@@ -305,8 +297,12 @@ class Journal:
             self.frame.show_save_error_dialog(exit_imminent)
             return True
 
-        something_saved = storage.save_months_to_disk(self.months,
-            self.dirs.data_dir, self.frame, exit_imminent, saveas)
+        try:
+            something_saved = storage.save_months_to_disk(
+                self.months, self.dirs.data_dir, exit_imminent, saveas)
+        except (IOError, OSError):
+            self.frame.show_save_error_dialog(exit_imminent)
+            something_saved = None
 
         if something_saved:
             self.show_message(_('The content has been saved to %s') % self.dirs.data_dir, error=False)
@@ -415,7 +411,6 @@ class Journal:
 
         if not Month.same_month(new_date, old_date) or self.month is None:
             self.month = self.get_month(self.date)
-            #self.month.visited = True
 
         self.frame.set_date(self.month, self.date, self.day)
 
@@ -441,6 +436,11 @@ class Journal:
 
 
     def change_date(self, new_date):
+        if new_date < datetime.date(1900, 1, 1):
+            self.show_message('Only dates after 1900 are supported.',
+                              title='Too Early', error=True)
+            return
+
         if new_date == self.date:
             return
 
@@ -467,14 +467,16 @@ class Journal:
     def show_message(self, msg, title=None, error=False):
         if error and not title:
             title = _('Error')
+
         if error:
             msg_type = gtk.MESSAGE_ERROR
             log_level = logging.ERROR
         else:
             msg_type = gtk.MESSAGE_INFO
             log_level = logging.INFO
+
         self.frame.show_message(title, msg, msg_type)
-        logging.log(log_level, '%s. %s' % (title, msg))
+        logging.log(log_level, '%s. %s' % (title, msg) if title else msg)
 
 
     @property
@@ -484,7 +486,7 @@ class Journal:
 
 
     def normalize_tag(self, tag):
-        return tag.replace(' ', '').lower()
+        return tag.replace(' ', '_').lower()
 
 
     def get_entries(self, category):
