@@ -17,15 +17,13 @@
 # 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 # -----------------------------------------------------------------------
 
-from __future__ import with_statement
-
 import logging
 import os
 import re
 import sys
 
-import pango
 import gobject
+import pango
 
 # Testing
 if __name__ == '__main__':
@@ -34,12 +32,26 @@ if __name__ == '__main__':
                     format='%(levelname)-8s %(message)s',)
 
 from rednotebook.external import txt2tags
+from rednotebook.data import HASHTAG
+from rednotebook.util import filesystem
 
 
 # Linebreaks are only allowed at line ends
 REGEX_LINEBREAK = r'\\\\[\s]*$'
 REGEX_HTML_LINK = r'<a.*?>(.*?)</a>'
 
+# pic [""/home/user/Desktop/RedNotebook pic"".png]
+PIC_NAME = r'\S.*?\S|\S'
+PIC_EXT = r'(?:png|jpe?g|gif|eps|bmp)'
+REGEX_PIC = re.compile(r'(\["")(%s)("")(\.%s)(\?\d+)?(\])' % (PIC_NAME, PIC_EXT), flags=re.I)
+
+# named local link [my file.txt ""file:///home/user/my file.txt""]
+# named link in web [heise ""http://heise.de""]
+REGEX_NAMED_LINK = re.compile(r'(\[)(.*?)(\s"")(\S.*?\S)(""\])', flags=re.I | re.L)
+
+ESCAPE_COLOR = r'XBEGINCOLORX\1XSEPARATORX\2XENDCOLORX'
+COLOR_ESCAPED = r'XBEGINCOLORX(.*?)XSEPARATORX(.*?)XENDCOLORX'
+
 TABLE_HEAD_BG = '#aaa'
 
 CHARSET_UTF8 = '<meta http-equiv="Content-Type" content="text/html; charset=utf-8" />'
@@ -47,7 +59,12 @@ CHARSET_UTF8 = '<meta http-equiv="Content-Type" content="text/html; charset=utf-
 CSS = """\
 <style type="text/css">
     body {
-        font-family: Ubuntu, Helvetica, Arial, sans-serif;
+        font-family: %(font)s;
+    }
+    <!-- Don't split last line between pages.
+         This fix is only supported by Opera -->
+    p {
+        page-break-inside: avoid;
     }
     blockquote {
         margin: 1em 2em;
@@ -74,7 +91,7 @@ CSS = """\
         text-align: left;
         padding-top: 5px;
         padding-bottom: 4px;
-        background-color: %(TABLE_HEAD_BG)s;
+        background-color: %(table_head_bg)s;
         color: #ffffff;
     }
     hr.heavy {
@@ -82,25 +99,25 @@ CSS = """\
         background-color: black;
     }
 </style>
-""" % globals()
+"""
 
 # MathJax
-MATHJAX_FILE = '/usr/share/javascript/mathjax/MathJax.js'
 FORMULAS_SUPPORTED = True
-if not os.path.isfile(MATHJAX_FILE):
-    MATHJAX_FILE = 'http://cdn.mathjax.org/mathjax/latest/MathJax.js'
-logging.info('MathJax location: %s' % MATHJAX_FILE)
+MATHJAX_FILE = 'http://cdn.mathjax.org/mathjax/latest/MathJax.js'
+MATHJAX_FINISHED = 'MathJax finished'
 
+# Explicitly setting inlineMath: [ ['\\(','\\)'] ] doesn't work.
+# Using defaults:
+#       displayMath: [ ['$$','$$'], ['\[','\]'] ]
+#       inlineMath:  [['\(','\)']]
+MATHJAX_DELIMITERS = ['$$', '\\(', '\\)', r'\\[', '\\]']
 MATHJAX = """\
 <script type="text/x-mathjax-config">
   MathJax.Hub.Config({
+  messageStyle: "none",
   config: ["MMLorHTML.js"],
   jax: ["input/TeX","input/MathML","output/HTML-CSS","output/NativeMML"],
-  tex2jax: {
-      inlineMath: [ ['$','$'] ],
-      displayMath: [ ['$$','$$'] ],
-      processEscapes: true
-    },
+  tex2jax: {},
   extensions: ["tex2jax.js","mml2jax.js","MathMenu.js","MathZoom.js"],
   TeX: {
     extensions: ["AMSmath.js","AMSsymbols.js","noErrors.js","noUndefined.js"]
@@ -110,7 +127,13 @@ MATHJAX = """\
 <script type="text/javascript"
   src="%s">
 </script>
-""" % (MATHJAX_FILE)
+<script>
+  MathJax.Hub.Queue(function () {
+    document.title = "%s";
+    document.title = "RedNotebook";
+  });
+</script>
+""" % (MATHJAX_FILE, MATHJAX_FINISHED)
 
 
 def convert_categories_to_markup(categories, with_category_title=True):
@@ -128,7 +151,7 @@ def convert_categories_to_markup(categories, with_category_title=True):
     return markup
 
 
-def get_markup_for_day(day, with_text=True, categories=None, date=None):
+def get_markup_for_day(day, with_text=True, with_tags=True, categories=None, date=None):
     '''
     Used for exporting days
     '''
@@ -145,11 +168,11 @@ def get_markup_for_day(day, with_text=True, categories=None, date=None):
     # Add Categories
     category_content_pairs = day.get_category_content_pairs()
 
-    if categories:
+    if with_tags and categories:
         categories = [word.lower() for word in categories]
-        export_categories = dict((x,y) for (x, y) in category_content_pairs.items()
-                        if x.lower() in categories)
-    elif categories is None:
+        export_categories = dict((x, y) for (x, y) in category_content_pairs.items()
+                                 if x.lower() in categories)
+    elif with_tags and categories is None:
         # No restrictions
         export_categories = category_content_pairs
     else:
@@ -172,13 +195,16 @@ def get_markup_for_day(day, with_text=True, categories=None, date=None):
     return ''
 
 
-def _get_config(type):
+def _get_config(target, options):
 
     config = {}
 
     # Set the configuration on the 'config' dict.
     config = txt2tags.ConfigMaster()._get_defaults()
 
+    config['outfile'] = txt2tags.MODULEOUT  # results as list
+    config['target'] = target
+
     # The Pre (and Post) processing config is a list of lists:
     # [ [this, that], [foo, bar], [patt, replace] ]
     config['postproc'] = []
@@ -187,7 +213,13 @@ def _get_config(type):
     # Allow line breaks, r'\\\\' are 2 \ for regexes
     config['preproc'].append([REGEX_LINEBREAK, 'LINEBREAK'])
 
-    if type == 'xhtml' or type == 'html':
+    # Highlight hashtags.
+    config['preproc'].append([HASHTAG.pattern, r'\1{\2\3|color:red}'])
+
+    # Escape color markup.
+    config['preproc'].append([r'\{(.*?)\|color:(.+?)\}', ESCAPE_COLOR])
+
+    if target in ['xhtml', 'html']:
         config['encoding'] = 'UTF-8'  # document encoding
         config['toc'] = 0
         config['css-sugar'] = 1
@@ -195,16 +227,16 @@ def _get_config(type):
         # Fix encoding for export opened in firefox
         config['postproc'].append([r'<head>', '<head>' + CHARSET_UTF8])
 
-        # Custom css
-        config['postproc'].append([r'</head>', CSS + '</head>'])
-
         # Line breaks
         config['postproc'].append([r'LINEBREAK', '<br />'])
 
         # Apply image resizing
         config['postproc'].append([r'src=\"WIDTH(\d+)-', r'width="\1" src="'])
 
-    elif type == 'tex':
+        # {{red text|color:red}} -> <span style="color:red">red text</span>
+        config['postproc'].append([COLOR_ESCAPED, r'<span style="color:\2">\1</span>'])
+
+    elif target == 'tex':
         config['encoding'] = 'utf8'
         config['preproc'].append(['€', 'Euro'])
 
@@ -233,13 +265,17 @@ def _get_config(type):
         # Apply image resizing
         config['postproc'].append([r'includegraphics\{("?)WIDTH(\d+)-', r'includegraphics[width=\2px]{\1'])
 
-        # We want the plain latex formulas unescaped
+        # We want the plain latex formulas unescaped.
+        # Allowed formulas: $$...$$, \[...\], \(...\)
+        config['preproc'].append([r'\\\[\s*(.+?)\s*\\\]', r"BEGINEQUATION''\1''ENDEQUATION"])
         config['preproc'].append([r'\$\$\s*(.+?)\s*\$\$', r"BEGINEQUATION''\1''ENDEQUATION"])
         config['postproc'].append([r'BEGINEQUATION(.+)ENDEQUATION', r'$$\1$$'])
-        config['preproc'].append([r'\$\s*(.+?)\s*\$', r"BEGINMATH''\1''ENDMATH"])
+        config['preproc'].append([r'\\\(\s*(.+?)\s*\\\)', r"BEGINMATH''\1''ENDMATH"])
         config['postproc'].append([r'BEGINMATH(.+)ENDMATH', r'$\1$'])
 
-    elif type == 'txt':
+        config['postproc'].append([COLOR_ESCAPED, r'\\textcolor{\2}{\1}'])
+
+    elif target == 'txt':
         # Line breaks
         config['postproc'].append([r'LINEBREAK', '\n'])
 
@@ -252,19 +288,77 @@ def _get_config(type):
     img_name = r'\S.*\S|\S'
 
     # Apply this prepoc only after the latex image quotes have been added
-    # TODO: Allow whitespace around [ and ]
     config['preproc'].append([r'\[(%s\.(%s))\?(\d+)\]' % (img_name, img_ext), r'[WIDTH\3-\1]'])
 
+    # Disable colors for all other targets.
+    config['postproc'].append([COLOR_ESCAPED, r'\1'])
+
+    # MathJax
+    if options.pop('add_mathjax'):
+        config['postproc'].append([r'</body>', MATHJAX + '</body>'])
+
+    # Custom css
+    fonts = options.pop('font', 'sans-serif')
+    if 'html' in target:
+        css = CSS % {'font': fonts, 'table_head_bg': TABLE_HEAD_BG}
+        config['postproc'].append([r'</head>', css + '</head>'])
+
+    config.update(options)
+
     return config
 
 
-def convert(txt, target, headers=None, options=None):
+def _convert_paths(txt, data_dir):
+    def _convert_uri(uri):
+        path = uri[len('file://'):] if uri.startswith('file://') else uri
+        # Check if relative file exists and convert it if it does.
+        if (not any(uri.startswith(proto) for proto in filesystem.REMOTE_PROTOCOLS) and
+                not os.path.isabs(path)):
+            path = os.path.join(data_dir, path)
+            assert os.path.isabs(path), path
+            if os.path.exists(path):
+                uri = filesystem.get_local_url(path)
+        return uri
+
+    def _convert_pic_path(match):
+        uri = _convert_uri(match.group(2) + match.group(4))
+        # Reassemble picture markup.
+        name, ext = os.path.splitext(uri)
+        parts = [match.group(1), name, match.group(3), ext]
+        if match.group(5) is not None:
+            parts.append(match.group(5))
+        parts.append(match.group(6))
+        return ''.join(parts)
+
+    def _convert_file_path(match):
+        uri = _convert_uri(match.group(4))
+        # Reassemble link markup
+        parts = [match.group(i) for i in range(1, 6)]
+        parts[3] = uri
+        return ''.join(parts)
+
+    txt = REGEX_PIC.sub(_convert_pic_path, txt)
+    txt = REGEX_NAMED_LINK.sub(_convert_file_path, txt)
+    return txt
+
+
+def convert(txt, target, data_dir, headers=None, options=None):
     '''
     Code partly taken from txt2tags tarball
     '''
-    add_mathjax = FORMULAS_SUPPORTED and 'html' in target and '$' in txt
+    options = options or {}
 
-    # Here is the marked body text, it must be a list.
+    # Only add MathJax code if there is a formula.
+    options['add_mathjax'] = (
+        FORMULAS_SUPPORTED and
+        'html' in target and
+        any(x in txt for x in MATHJAX_DELIMITERS))
+    logging.debug('Add mathjax code: %s' % options['add_mathjax'])
+
+    # Turn relative paths into absolute paths.
+    txt = _convert_paths(txt, data_dir)
+
+    # The body text must be a list.
     txt = txt.split('\n')
 
     # Set the three header fields
@@ -275,38 +369,30 @@ def convert(txt, target, headers=None, options=None):
         else:
             headers = ['', '', '']
 
-    config = _get_config(target)
-
-    # MathJax
-    if add_mathjax:
-        config['postproc'].append([r'</body>', MATHJAX + '</body>'])
-
-    config['outfile'] = txt2tags.MODULEOUT  # results as list
-    config['target'] = target
-
-    if options is not None:
-        config.update(options)
+    config = _get_config(target, options)
 
     # Let's do the conversion
     try:
-        headers   = txt2tags.doHeader(headers, config)
+        headers = txt2tags.doHeader(headers, config)
         body, toc = txt2tags.convert(txt, config)
-        footer  = txt2tags.doFooter(config)
+        footer = txt2tags.doFooter(config)
         toc = txt2tags.toc_tagger(toc, config)
         toc = txt2tags.toc_formatter(toc, config)
-        full_doc  = headers + toc + body + footer
-        finished  = txt2tags.finish_him(full_doc, config)
+        full_doc = headers + toc + body + footer
+        finished = txt2tags.finish_him(full_doc, config)
         result = '\n'.join(finished)
-
     # Txt2tags error, show the messsage to the user
     except txt2tags.error, msg:
         logging.error(msg)
         result = msg
-
     # Unknown error, show the traceback to the user
     except:
-        result = txt2tags.getUnknownErrorMessage()
-        logging.error(result)
+        result = ('<b>Error</b>: This day contains invalid '
+            '<a href="http://txt2tags.org/markup.html">txt2tags markup</a>. '
+            'You can help us fix this by submitting a bugreport in the '
+            '<a href="https://code.google.com/p/txt2tags/issues/list">'
+            'txt2tags bugtracker</a>. Please append the day\'s text to the issue.')
+        logging.error('Invalid markup:\n%s' % txt2tags.getUnknownErrorMessage())
     return result
 
 
@@ -343,8 +429,8 @@ def convert_to_pango(txt, headers=None, options=None):
     # Let's do the conversion
     try:
         body, toc = txt2tags.convert(txt, config)
-        full_doc  = body
-        finished  = txt2tags.finish_him(full_doc, config)
+        full_doc = body
+        finished = txt2tags.finish_him(full_doc, config)
         result = ''.join(finished)
 
     # Txt2tags error, show the messsage to the user
@@ -382,14 +468,15 @@ def convert_to_pango(txt, headers=None, options=None):
 
 def convert_from_pango(pango_markup):
     original_txt = pango_markup
-    replacements = dict((('<b>', '**'), ('</b>', '**'),
-                        ('<i>', '//'), ('</i>', '//'),
-                        ('<s>', '--'), ('</s>', '--'),
-                        ('<u>', '__'), ('</u>', '__'),
-                        ('&amp;', '&'),
-                        ('&lt;', '<'), ('&gt;', '>'),
-                        ('\n', r'\\'),
-                        ))
+    replacements = dict((
+        ('<b>', '**'), ('</b>', '**'),
+        ('<i>', '//'), ('</i>', '//'),
+        ('<s>', '--'), ('</s>', '--'),
+        ('<u>', '__'), ('</u>', '__'),
+        ('&amp;', '&'),
+        ('&lt;', '<'), ('&gt;', '>'),
+        ('\n', r'\\'),
+    ))
     for orig, repl in replacements.items():
         pango_markup = pango_markup.replace(orig, repl)
 
@@ -401,5 +488,5 @@ def convert_from_pango(pango_markup):
 if __name__ == '__main__':
     from rednotebook.util.utils import show_html_in_browser
     markup = 'Aha\n\tThis is a quote. It looks very nice. Even with many lines'
-    html = convert(markup, 'xhtml')
+    html = convert(markup, 'xhtml', '/tmp')
     show_html_in_browser(html, '/tmp/test.html')
