@@ -17,16 +17,13 @@
 # 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 # -----------------------------------------------------------------------
 
-from __future__ import with_statement, division
-
 import signal
 import os
 import re
-from urllib2 import urlopen, URLError
+import httplib
+from urllib2 import urlopen
 import webbrowser
 import logging
-from optparse import IndentedHelpFormatter
-import textwrap
 from distutils.version import StrictVersion
 
 import gtk
@@ -43,7 +40,7 @@ def set_environment_variables(config):
     variables = {}
 
     for variable, value in variables.iteritems():
-        if not variable in os.environ:
+        if variable not in os.environ:
             # Only add environment variable if it does not exist yet
             os.environ[variable] = config.read(variable, default=value)
             logging.info('%s set to %s' % (variable, value))
@@ -63,50 +60,30 @@ def setup_signal_handlers(journal):
     SIGKILL cannot be caught
     SIGINT is caught again by KeyboardInterrupt
     """
-
     signals = []
-
-    try:
-        signals.append(signal.SIGHUP)  #Terminal closed, Parent process dead
-    except AttributeError:
-        pass
-    try:
-        signals.append(signal.SIGINT)  #Interrupt from keyboard (CTRL-C)
-    except AttributeError:
-        pass
-    try:
-        signals.append(signal.SIGQUIT) #Quit from keyboard
-    except AttributeError:
-        pass
-    try:
-        signals.append(signal.SIGABRT) #Abort signal from abort(3)
-    except AttributeError:
-        pass
-    try:
-        signals.append(signal.SIGTERM) #Termination signal
-    except AttributeError:
-        pass
-    try:
-        signals.append(signal.SIGTSTP) #Stop typed at tty
-    except AttributeError:
-        pass
-
+    signal_names = [
+        'SIGHUP',   # Terminal closed, Parent process dead
+        'SIGINT',   # Interrupt from keyboard (CTRL-C)
+        'SIGQUIT',  # Quit from keyboard
+        'SIGABRT',  # Abort signal from abort(3)
+        'SIGTERM',  # Termination signal
+        'SIGTSTP',  # Stop typed at tty
+    ]
 
     def signal_handler(signum, frame):
         logging.info('Program was abnormally aborted with signal %s' % signum)
         journal.exit()
 
+    for signal_name in signal_names:
+        signal_number = getattr(signal, signal_name, None)
+        if signal_number is not None:
+            try:
+                signal.signal(signal_number, signal_handler)
+                signals.append(signal_number)
+            except RuntimeError:
+                logging.info('Could not connect signal number %d' % signal_number)
 
-    msg = 'Connected Signals: '
-
-    for signal_number in signals:
-        try:
-            msg += str(signal_number) + ' '
-            signal.signal(signal_number, signal_handler)
-        except RuntimeError:
-            msg += '\n_false Signal Number: ' + signal_number
-
-    logging.info(msg)
+    logging.info('Connected Signals: %s' % signals)
 
 
 def get_new_version_number():
@@ -121,30 +98,32 @@ def get_new_version_number():
         if not match:
             return None
         new_version = match.group(1)
+        new_version = StrictVersion(new_version)
         logging.info('%s is the latest version' % new_version)
         return new_version
-    except URLError:
+    except (IOError, httplib.HTTPException):
         return None
 
 
 def check_new_version(journal, current_version, startup=False):
+    current_version = StrictVersion(current_version)
     new_version = get_new_version_number()
 
-    if new_version:
-        new_version = StrictVersion(new_version)
+    if new_version is not None:
+        newer_version_available = new_version > current_version
     else:
         logging.error('New version info could not be read')
-        new_version = 'unknown'
+        new_version = _('unknown')
+        newer_version_available = None
 
-    current_version = StrictVersion(current_version)
-    # Only compare versions if new version could be read
-    newer_version_available = ((new_version > current_version)
-                    if isinstance(new_version, StrictVersion) else True)
-    logging.info('A newer version is available: %s' % newer_version_available)
+    logging.info('Current version: %s, latest version: %s, newer: %s' %
+                 (current_version, new_version, newer_version_available))
 
     if newer_version_available or not startup:
         dialog = gtk.MessageDialog(parent=None, flags=gtk.DIALOG_MODAL,
-            type=gtk.MESSAGE_INFO, buttons=gtk.BUTTONS_YES_NO, message_format=None)
+                                   type=gtk.MESSAGE_INFO,
+                                   buttons=gtk.BUTTONS_YES_NO,
+                                   message_format=None)
         dialog.set_transient_for(journal.frame.main_frame)
         primary_text = (_('You have version <b>%s</b>.') % current_version + ' ' +
                         _('The latest version is <b>%s</b>.') % new_version)
@@ -195,74 +174,5 @@ class StreamDuplicator(object):
             stream.flush()
 
     def close(self):
-       for stream in self.streams():
-           self.stream.close()
-
-
-
-class IndentedHelpFormatterWithNL(IndentedHelpFormatter):
-    """
-    Code taken from "Dan"
-    http://groups.google.com/group/comp.lang.python/browse_frm/thread/e72deee779d9989b/
-
-    This class preserves newlines in the optparse help
-    """
-    def format_description(self, description):
-        if not description: return ""
-        desc_width = self.width - self.current_indent
-        indent = " "*self.current_indent
-        # the above is still the same
-        bits = description.split('\n')
-        formatted_bits = [
-            textwrap.fill(bit,
-                desc_width,
-                initial_indent=indent,
-                subsequent_indent=indent)
-            for bit in bits]
-        result = "\n".join(formatted_bits) + "\n"
-        return result
-
-    def format_option(self, option):
-        # The help for each option consists of two parts:
-        #    * the opt strings and metavars
-        #    eg. ("-x", or "-f FILENAME, --file=FILENAME")
-        #    * the user-supplied help string
-        #    eg. ("turn on expert mode", "read data from FILENAME")
-        #
-        # If possible, we write both of these on the same line:
-        #    -x     turn on expert mode
-        #
-        # But if the opt string list is too long, we put the help
-        # string on a second line, indented to the same column it would
-        # start in if it fit on the first line.
-        #    -f FILENAME, --file=FILENAME
-        #            read data from FILENAME
-        result = []
-        opts = self.option_strings[option]
-        opt_width = self.help_position - self.current_indent - 2
-        if len(opts) > opt_width:
-            opts = "%*s%s\n" % (self.current_indent, "", opts)
-            indent_first = self.help_position
-        else: # start help on same line as opts
-            opts = "%*s%-*s " % (self.current_indent, "", opt_width, opts)
-            indent_first = 0
-        result.append(opts)
-        if option.help:
-            help_text = option.help
-            # Everything is the same up through here
-            help_lines = []
-            help_text = "\n".join([x.strip() for x in
-                                help_text.split("\n")])
-            for para in help_text.split("\n\n"):
-                help_lines.extend(textwrap.wrap(para, self.help_width))
-                if len(help_lines):
-                    # for each paragraph, keep the double newlines..
-                    help_lines[-1] += "\n"
-                    # Everything is the same after here
-            result.append("%*s%s\n" % (
-                indent_first, "", help_lines[0]))
-            result.extend(["%*s%s\n" % (self.help_position, "", line)
-                for line in help_lines[1:]])
-        elif opts[-1] != "\n":
-            result.append("\n")
-        return "".join(result)
+        for stream in self.streams():
+            self.stream.close()
